M <- 1e3
plus1 <- ""

ggQQ <- function(LM) # argument: a linear model
{
    y <- quantile(LM$resid[!is.na(LM$resid)], c(0.25, 0.75))
    x <- qnorm(c(0.25, 0.75))
    slope <- diff(y)/diff(x)
    int <- y[1L] - slope * x[1L]
    p <- ggplot(LM, aes(sample=.resid)) +
        stat_qq(alpha = 0.5) +
        geom_abline(slope = slope, intercept = int, color="blue")

    return(p)
}

shinyServer(function(input, output,session) {

    gotLibs <- reactive({
        a<-is.element("ggplot2", installed.packages()[,1])
        b<-is.element("grid", installed.packages()[,1])
        x <- c(a,b,a&b)
        names(x) <- c("ggplot2","grid","both")
        x
    })
    output$checkGraph <- renderText({
        if(input$whichgraph=="base") return("")
        if(gotLibs()[1]) library(ggplot2)
        if(gotLibs()[2]) library(grid)    
        if(!gotLibs()[3]) return(paste(red,"<h4>Warning: some R libraries not found, will use base graphics</h4>"))
        NULL       
    }) 

    data <- reactive({        
          if(input$dataset == "Bernoulli") 
              x <- (rbinom(M,input$n,input$p)-input$n*input$p)/sqrt(input$n*input$p*(1-input$p))
          if(input$dataset == "Exponential")     
              x <- (apply(matrix(rexp(M*input$n,1),nrow=M),1,mean)-1)*sqrt(input$n)
          if(input$dataset == "t")     
              x <- apply(matrix(rt(M*input$n,4),nrow=M),1,mean)*sqrt(input$n)/sqrt(2)
          x    

    })             

   output$table1 <- renderText({
        x <- data()
        A <- matrix(0,13,3)
        A[,1] <- c(-3,-2.5,-2,-1.5,-1,-0.5,0,0.5,1,1.5,2,2.4,3)
        for(i in 1:13) {
            z <- A[i,1]
            if(input$dataset == "Bernoulli") z <- z+0.5/sqrt(input$n)
            A[i,2] <- round(length(x[x<z])/length(x),3)
        }    
        A[,3] <- round(pnorm(A[,1]),3)
        tbl0 <- "<h4>Probabilities of the form P(X&le;x)</h4>"   
        tbl1 <- paste("<table border>")
        tra <- paste("<Tr align=\"right\">")
        tbl2 <- paste(tra,"<TH>",plus1,"x &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</TH><TH>",plus1,"Sample Mean&nbsp;</TH><TH>",plus1,"Standard Normal</TH><TH>",plus1,"Difference</TH></TR>")
        tbl <- rep("",13)
        for(i in 1:13) 
              tbl[i] <-paste(tra,"<Td><strong>",plus1,A[i,1],"</Td><TD><strong>"
              ,plus1,A[i,2],"</Td><TD><strong>",plus1,A[i,3],"</TD><TD><strong>",plus1,round(A[i,3]-A[i,2],3),"</td></TR>")
            
        c(tbl0,tbl1,tbl2,tbl,"</table>")            
   })    
   output$text2 <- renderText({
      paste("<h3>Mean of n =",input$n," observations</h3>") 
   })
                
   output$plot1 <- renderPlot({
        if(input$whichgraph=="base" | !gotLibs()[3]) {
            par(mfrow=c(2,2))
            hist(data(),100,xlab="Sample Means",main="",ylab="")
            qqnorm(data())
            boxplot(data(),horizontal=T)
        }
        else {
             dta <- data.frame(x=data(),y=rep(0,length(data())))
             plt1 <- ggplot(dta,aes(x))+geom_histogram(colour = "black", fill = "white")
             plt2 <- ggplot(dta, aes(sample=x)) +stat_qq()
             plt3 <- ggplot(dta,aes(y,x))+geom_boxplot()+coord_flip()
             pushViewport(viewport(layout = grid.layout(2, 2)))
             print(plt1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
             print(plt2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))        
             print(plt3, vp = viewport(layout.pos.row = 2, layout.pos.col = 1))        
        }    
   })
           
   output$text1 <- renderText({ 
         if(input$dataset == "Bernoulli")
            return("<h4>Bernoulli is a discrete random variable, so the graphs look a little strange!</h4>")        
   })

   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#clt\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 11/29/2014</h6>")
      
  })  
   output$explainText <- renderText({ 
         "<h5>Explanation: This demo illustrates the Central Limit Theorem </h5>"        
   })

})

